# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/dreamai_pdf',
                'doc_host': 'https://HamzaFarhan.github.io',
                'git_url': 'https://github.com/HamzaFarhan/dreamai_pdf',
                'lib_path': 'dreamai_pdf'},
  'syms': { 'dreamai_pdf.core': { 'dreamai_pdf.core.cid_to_char': ('core.html#cid_to_char', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.extract_text': ('core.html#extract_text', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.extract_text_dict': ('core.html#extract_text_dict', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.extract_text_list': ('core.html#extract_text_list', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.process_line': ('core.html#process_line', 'dreamai_pdf/core.py'),
                                  'dreamai_pdf.core.process_text': ('core.html#process_text', 'dreamai_pdf/core.py')},
            'dreamai_pdf.extract': { 'dreamai_pdf.extract.load_ems_model': ('extract.html#load_ems_model', 'dreamai_pdf/extract.py'),
                                     'dreamai_pdf.extract.load_segs_model': ('extract.html#load_segs_model', 'dreamai_pdf/extract.py'),
                                     'dreamai_pdf.extract.print_segments': ('extract.html#print_segments', 'dreamai_pdf/extract.py'),
                                     'dreamai_pdf.extract.text_to_segments': ('extract.html#text_to_segments', 'dreamai_pdf/extract.py'),
                                     'dreamai_pdf.extract.write_embeddings': ('extract.html#write_embeddings', 'dreamai_pdf/extract.py'),
                                     'dreamai_pdf.extract.write_segments': ('extract.html#write_segments', 'dreamai_pdf/extract.py')},
            'dreamai_pdf.imports': {},
            'dreamai_pdf.ner': { 'dreamai_pdf.ner.edu_ner': ('ner.html#edu_ner', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.is_valid_jner': ('ner.html#is_valid_jner', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.is_valid_tner': ('ner.html#is_valid_tner', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.job_ner': ('ner.html#job_ner', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.load_job_model': ('ner.html#load_job_model', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.load_ner_model': ('ner.html#load_ner_model', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.proc_ner': ('ner.html#proc_ner', 'dreamai_pdf/ner.py'),
                                 'dreamai_pdf.ner.work_ner': ('ner.html#work_ner', 'dreamai_pdf/ner.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_extract.ipynb.

# %% auto 0
__all__ = ['print_segments', 'text_to_segments', 'load_segs_model', 'load_ems_model', 'write_segments', 'write_embeddings']

# %% ../nbs/01_extract.ipynb 2
from .imports import *
from .core import *


# %% ../nbs/01_extract.ipynb 3
def print_segments(segments: dict, limit: int = 10, width: int = 100):
    for k, v in segments.items():
        print(f"{k}: {len(v)}")
        for s in v[:limit]:
            print("\t", end="")
            pprint(s, width=width)
            print()
        print("-" * width + 4)


def text_to_segments(
    text: Union[str, List[str]],
    segs_model: SetFitModel,
    thresh: float = 0.6,
    classes: List = ["Work Experience", "Education", "Certifications", "Other"],
):
    preds = segs_model.predict(text).detach().cpu().numpy()
    probs = segs_model.predict_proba(text).detach().cpu().numpy().max(1)
    preds[probs < thresh] = 3
    pred_classes = [classes[p] for p in preds]
    segments = {pc: [] for pc in classes}
    for pc, txt in zip(pred_classes, text):
        if txt not in segments[pc]:
            segments[pc].append(txt)
    return segments, pred_classes, probs


def load_segs_model(
    model_name: str = "HamzaFarhan/PDFSegs", device: Union[str, torch.device] = "cpu"
):
    return SetFitModel.from_pretrained(model_name).to(device)


def load_ems_model(
    model_name: str = "HamzaFarhan/PDFSegs", device: Union[str, torch.device] = "cpu"
):
    return SentenceTransformer(model_name, device=device)


def write_segments(segs_model, data_path, output_path="pdf_segments", n_lines=2):
    """
    Extracts text from PDFs and writes segments to JSON files.

    Parameters
    ----------
    segs_model : SetFitModel
        SetFit model for segment classification.
    data_path : Union[str, Path, List[Union[str, Path]]]
        Path to PDFs. Can be a single file, a directory, or a list of files/directories.
    output_path : Union[str, Path]
        Folder to write JSON files. Defaults to 'pdf_segments'.
    n_lines : int
        Number of lines to group together when extracting text from PDFs.
    Returns
    -------
    None
    """
    output_path = Path(output_path)
    os.makedirs(output_path, exist_ok=True)
    pdfs = resolve_data_path(data_path)
    text_dict = extract_text_dict(pdfs, n_lines=n_lines)
    for file in pdfs:
        try:
            pdf_text = text_dict[str(file)]
            (segments,) = text_to_segments(pdf_text, segs_model, thresh=0.6)[0]
            fn = (output_path / Path(file).stem).with_suffix(".json")
            with open(fn, "w") as f:
                json.dump(segments, f, indent=4)
        except Exception as e:
            msg.fail(f"\nCould not write segments for file: {str(file)}", e)
            # print(f"\nCould not write segments for file: {str(file)}\n{e}")


def write_embeddings(
    segs_model, ems_model, data_path, output_path="pdf_ems", n_lines=3
):
    """
    Extracts text from PDFs and writes embeddings to JSON files.

    Parameters
    ----------
    segs_model : SetFitModel
        SetFit model for segment classification.
    ems_model : SentenceTransformer
        SentenceTransformer model for embedding generation.
    data_path : Union[str, Path, List[Union[str, Path]]]
        Path to PDFs. Can be a single file, a directory, or a list of files/directories.
    output_path : Union[str, Path]
        Folder to write JSON files. Defaults to 'pdf_ems'.
    n_lines : int
        Number of lines to group together when extracting text from PDFs.
    Returns
    -------
    None
    """
    os.makedirs(output_path, exist_ok=True)
    seg_ids = {"Work Experience": 1, "Education": 2, "Certifications": 3, "Other": 4}
    pdfs = resolve_data_path(data_path)
    text_dict = extract_text_dict(pdfs, n_lines=n_lines)
    for file in pdfs:
        try:
            fn = Path(file).stem
            pdf_text = text_dict[str(file)]
            ems = ems_model.encode(pdf_text)
            pred_classes = text_to_segments(pdf_text, segs_model)[1]
            for i, data in enumerate(zip(pred_classes, ems)):
                pc, em = data
                seg_id = seg_ids[pc]
                jf = Path(output_path) / f"{fn}_{seg_id}_{i+1}.json"
                em_dict = {"id": fn, "embedding": em.tolist()}
                with open(jf, "w") as f:
                    json.dump(em_dict, f)
        except Exception as e:
            msg.fail(f"\nCould not write embeddings for {str(file)}", e)
            # print(f"\nCould not write embeddings for {str(file)}\n{e}")




# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/02_ner.ipynb.

# %% auto 0
__all__ = ['load_ner_model', 'load_job_model', 'proc_ner', 'job_ner', 'edu_ner', 'work_ner', 'is_valid_jner', 'is_valid_tner']

# %% ../nbs/02_ner.ipynb 2
from .imports import *
from .core import *
from .extract import *


# %% ../nbs/02_ner.ipynb 3
def load_ner_model(model_name='tner/deberta-v3-large-ontonotes5', device='cpu'):
    tokenizer = AutoTokenizer.from_pretrained(model_name)
    model = AutoModelForTokenClassification.from_pretrained(model_name)
    return pipeline('ner', model=model, tokenizer=tokenizer, aggregation_strategy="simple", device=device)

def load_job_model(model_name='ismail-lucifer011/autotrain-job_all-903929564', device='cpu'):
    tokenizer = AutoTokenizer.from_pretrained(model_name)
    model = AutoModelForTokenClassification.from_pretrained(model_name)
    return pipeline('ner', model=model, tokenizer=tokenizer, aggregation_strategy="simple", device=device)


# def proc_ner(txt, ner, ner_dict={'institute':'', 'date':''}):
#     ner_dict = copy.deepcopy(ner_dict)
#     mapper = {'institute':['ORG', 'FAC', 'GPE', 'LOC'], 'company':['ORG', 'FAC', 'GPE', 'LOC'],
#               'role':['Job'], 'degree':['WORK_OF_ART'], 'date':['DATE']}
#     ner_dict['text'] = txt
#     for d in ner:
#         eg = d['entity_group']
#         w = ' ' + d['word'].strip()
#         for k,v in mapper.items():
#             if eg in v and ner_dict.get(k, None) is not None and '##' not in w:
#             # if eg in v and k in ner_dict and '##' not in w:
#                 ner_dict[k] = (ner_dict[k] + w).strip()
                
#                 # if eg == 'DATE':
#                 # elif len(ner_dict[k]) == 0:
#                 #     ner_dict[k] = (ner_dict[k] + w).strip()
#                 # else:
#                 #     ner_list.append(ner_dict)
#                 #     ner_dict = {k:'' for k in ner_dict.keys()}
#                 #     ner_dict['text'] = txt
                    
#     return ner_dict

def proc_ner(txt, ner, ner_dict={'institute':'', 'date':''}, thresh=3):
    ner_dict = copy.deepcopy(ner_dict)
    org_key = 'institute' if 'institute' in ner_dict else 'company'
    mapper = {'ORG':org_key, 'FAC':org_key, 'GPE':org_key, 'LOC':org_key, 'Job':'role', 'WORK_OF_ART':'degree', 'DATE':'date'}
    ner_dict['text'] = txt
    for d in ner:
        eg = d['entity_group']
        w = ' ' + d['word'].strip()
        k = mapper.get(eg, None)
        if k is not None and ner_dict.get(k, None) is not None and not w.startswith('##'):
            ner_dict[k] = (ner_dict[k] + w).strip()
    res = {k:v for k,v in ner_dict.items() if len(v) > thresh}
    if res.get(org_key, None) is None:
        return {}
    return res

def job_ner(txt, tner, jner):
    return tner(txt) + jner(txt)

def edu_ner(txt, tner, ner_dict={'institute':'', 'date':''}):
    ner = tner(txt)
    return proc_ner(txt, ner, ner_dict)

def work_ner(txt, tner, jner, ner_dict={'company':'', 'date':''}):
    ner = job_ner(txt, tner, jner)
    return proc_ner(txt, ner, ner_dict)

def is_valid_jner(ner, thresh=3):
    return ner.get('company', None) is not None
    # rc = len(ner['company'])
    # rl = len(ner.get('role', 'RANDOM ROLE'))
    # dl = len(ner.get('date', 'RANDOM DATE'))
    # return (rc > thresh) and dl > thresh# and rl > thresh

def is_valid_tner(ner, thresh=3):
    return ner.get('institute', None) is not None
    # if ner.get('institute', None) is None:
    #     return False
    # il = len(ner['institute'])
    # dl = len(ner.get('degree', 'RANDOM DEGREE'))
    # dl2 = len(ner.get('date', 'RANDOM DATE'))
    # return il > thresh and (dl > 0 or dl2 > 0)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['cid_to_char', 'process_text', 'process_line', 'extract_text', 'extract_text_list', 'extract_text_dict']

# %% ../nbs/00_core.ipynb 2
from .imports import *


# %% ../nbs/00_core.ipynb 3
def cid_to_char(cidx: str):
    try:
        return chr(int(re.findall(r"\(cid\:(\d+)\)", cidx)[0]) + 29)
    except:
        return cidx


def process_text(text: str):
    text = text.strip()
    text = cid_to_char(text)
    text = re.sub("\xa0", " ", text)
    text = re.sub(r"\uf0b7", " ", text)
    text = re.sub(r"\(cid:\d{0,3}\)", " ", text)
    text = re.sub(r"•", "", text)
    text = re.sub(r"●", "", text)
    text = re.sub(r"▪", "", text)
    text = re.sub(r"", "", text)
    text = re.sub(r"➢", "", text)
    text = re.sub(r"\u2b9a", "", text)
    text = re.sub(r"\u201c", "", text)
    text = re.sub(r"\u201d", "", text)
    text = re.sub(r"\u2013", " ", text)
    text = re.sub(r"\u2019", "'", text)
    text = re.sub(r"\u2018", "'", text)
    text = re.sub(r"\u00f4", " ", text)
    text = re.sub(r"\u00f6", "o", text)
    text = re.sub(r"\u00e9", "e", text)
    text = re.sub(r"\u00e8", "e", text)
    text = re.sub(r"\u00e7", " ", text)
    text = re.sub(r"\u00a7", "", text)
    text = re.sub(r"\u00e3", "a", text)
    text = re.sub(r"\uf0a7", "", text)
    text = re.sub(r"\uf076", "", text)
    text = re.sub(r"\u00ad", "", text)
    text = re.sub(r"\u00ab", "", text)
    text = re.sub(r"\u00bb", "", text)
    text = re.sub(r"\uf02d", "", text)
    text = re.sub(r"\uf0fc", "", text)
    text = re.sub(r"\uf06e", "", text)
    text = re.sub(r"\uf07a", "", text)
    text = re.sub(r"\ufb01", "fi", text)
    text = re.sub(r"\ufb00", "ff", text)
    text = re.sub(r"\uf0d8", "", text)
    text = re.sub(r"\u00b7", "", text)
    text = re.sub("\t", " ", text)
    text = re.sub(" +", " ", text)
    return text.strip()


def process_line(txt, n_lines=2):
    txt = process_text(txt)
    if not txt.isspace() and len(txt) > 2:
        if len(txt) > 512:
            nw = 10 * n_lines
            txt = txt.split()
            txt = [" ".join(txt[i : i + nw]) for i in range(0, len(txt), nw)]
        return txt
    return None


def extract_text(file, n_lines=3):
    if Path(file).suffix == ".pdf":
        try:
            pdf = PdfReader(str(file))
            pdf_text = []
            for pn, page in enumerate(pdf.pages):
                try:
                    text = page.extract_text().splitlines()
                    i = 0
                    end = i + n_lines
                    while end < len(text):
                        txt = " ".join(text[i:end])
                        txt = process_line(txt, n_lines=n_lines)
                        if txt is not None:
                            pdf_text.append(txt)

                        i += n_lines
                        end = i + n_lines
                    if end >= len(text):
                        txt = " ".join(text[i:])
                        txt = process_line(txt, n_lines=n_lines)
                        if txt is not None:
                            pdf_text.append(txt)
                except Exception as e:
                    msg.fail(
                        f"Could not extract text from page {pn} of file: {file}", f'\t{e}', spaced=True
                    )
        except Exception as e:
            msg.fail(f"Could not extract text from file: {file}", f'\t{e}', spaced=True)
            return []
        return flatten_list(pdf_text)
    else:
        return []


def extract_text_list(data_path, n_lines=2):
    try:
        pdfs = resolve_data_path(data_path)
    except Exception as e:
        msg.fail(f"Could not resolve data path: {data_path}", f'\t{e}', spaced=True)
        return []
    text_list = []
    for file in pdfs:
        if Path(file).suffix == ".pdf":
            try:
                pdf_text = extract_text(file, n_lines=n_lines)
                text_list.append(flatten_list(pdf_text))
            except Exception as e:
                msg.fail(f"Could not extract text from file: {file}", f'\t{e}', spaced=True)
    return text_list


def extract_text_dict(data_path, n_lines=2):
    try:
        pdfs = resolve_data_path(data_path)
    except Exception as e:
        msg.fail(f"Could not resolve data path: {data_path}", f'\t{e}', spaced=True)
        return []
    pdf_dict = {}
    for file in pdfs:
        if Path(file).suffix == ".pdf":
            try:
                pdf_text = extract_text(file, n_lines=n_lines)
                pdf_dict[str(file)] = flatten_list(pdf_text)
            except Exception as e:
                msg.fail(f"Could not extract text from file: {file}", f'\t{e}', spaced=True)
    return pdf_dict


# def extract_text_list(data_path, n_lines=2):
#     pdfs = resolve_data_path(data_path)
#     text_list = []
#     for file in pdfs:
#         if Path(file).suffix == ".pdf":
#             try:
#                 pdf = PdfReader(str(file))
#                 pdf_text = []
#                 for page in pdf.pages:
#                     text = page.extract_text().splitlines()
#                     i = 0
#                     end = i + n_lines
#                     while end < len(text):
#                         txt = " ".join(text[i:end])
#                         txt = process_line(txt, n_lines=n_lines)
#                         if txt is not None:
#                             pdf_text.append(txt)

#                         i += n_lines
#                         end = i + n_lines
#                     if end >= len(text):
#                         txt = " ".join(text[i:])
#                         txt = process_line(txt, n_lines=n_lines)
#                         if txt is not None:
#                             pdf_text.append(txt)
#                 text_list.append(flatten_list(pdf_text))
#             except Exception as e:
#                 msg.fail(f"Could not extract text from file: {file}\n{e}")
#     return text_list




from dreamai.core import *
from dreamai_dl.imports import *

from pypdf import PdfReader
from setfit import SetFitModel
from sentence_transformers import SentenceTransformer
from transformers import (
    AutoTokenizer,
    AutoModel,
    AutoModelForSequenceClassification,
    AutoModelForTokenClassification,
    pipeline,
)
